/*
 * Generic support for opening files.
 *
 * Copyright (C) 2021, 2022 Paul Boddie <paul@boddie.org.uk>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of
 * the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor,
 * Boston, MA  02110-1301, USA
 */

#pragma once

#include <fsserver/accessor.h>
#include <fsserver/directory_accessor.h>



/* An interface offering file opening support. */

class FileOpening
{
public:
    virtual bool accessing_directory(flags_t flags, fileid_t fileid) = 0;

    virtual bool accessing_file(flags_t flags, fileid_t fileid) = 0;

    virtual bool directory_is_empty(fileid_t fileid) = 0;

    virtual long get_fileid(const char *path, flags_t flags, fileid_t *fileid) = 0;

    virtual long get_parent(const char *path, fileid_t *fileid);

    virtual long make_accessor(flags_t flags, fileid_t fileid,
                               Accessor **accessor) = 0;

    virtual long make_directory_accessor(flags_t flags, fileid_t fileid,
                                         DirectoryAccessor **accessor) = 0;

    /* Filesystem object access and manipulation methods. */

    virtual long make_directory_object(const char *path, sys_mode_t mode) = 0;

    virtual long stat_object(const char *path, void *base, offset_t size) = 0;

    virtual long remove_object(fileid_t fileid) = 0;

    virtual long rename_object(const char *source, const char *target) = 0;

    virtual long unlink_object(fileid_t parent_fileid, fileid_t fileid) = 0;
};

// vim: tabstop=4 expandtab shiftwidth=4
